{*******************************************************************************
* TSortList                                                                    *
* ---------                                                                    *
* Class for operation with the sorted list. Uses the base TList class.         *
* Allows to add items in the sorted out order and to make fast search of items.*
* Are added 1 property and 3 new methods:                                      *
*------------------------------------------------------------------------------*
* Property: Compare - name of the function of comparing of a type              *
* TListSortCompare. (Look the description of a method Sort in TList).          *
* The name of the function should be assigned before execution AddSort and     *
* Search. If it is not made, is generated Error.                               *
*------------------------------------------------------------------------------*
* Methods: AddSort - allows to add items in List in sorted out order.          *
*          Search - realizes fast search of a item in sorted list.             *
*                  Item - pointer on required item. Can contain only "key"     *
*                         values, used in the function of comparing.           *
*                  Return - number of a item in the list, since 0.             *
*                  If the item is not found, comes back negative value (-1).   *
*          ClearAll - Clears memory of the pointers and memory, disrtibuted    *
*                     for Item                                                 *
* -----------------------------------------------------------------------------*
* Attention! In order to prevent of violation about sorting, do not use        *
* together with new AddSort by methods Add and Insert, stayed from TList.      *
*************************************** ****************************************
* Can be used without limitations.                                             *
*************************************** ****************************************
* Developer: Ivanov Yuri. E-mail: i7@mail.ru                                   *
* The information on other developments of the author can be looked on page    *
*  http: // i7.da.ru (Russian)                                                 *
*                                                                              *
* December, 2000 - July, 2002                                                  *
********************************************************************************
Example of use:
---------------
interface
...
type Tdat = record
     code: integer;
     txt: string[50];
     num: double;
end;
...
var
    ldat: TSortList;
    dat: ^Tdat;
...
implementation
//*********************************************
function Sort_dat(i1,i2: Pointer): integer;
var
d1,d2: ^Tdat;
begin
    d1:=i1; d2:=i2;
    if d1^.code < d2^.code then Result:=-1
    else
      if d1^.code > d2^.code then Result:=1
      else
        Result:=0;
end;
//*********************************************
procedure ....
var
 d: Tdat;
 pos: integer;
begin
...
   // Addition of a item
        New(dat);
        dat^.code:=codd;
        dat^.txt:=st;
        dat^.num:=dob;
        ldat.AddSort(dat);
    end;
...
  // Search of a item
        d.code:=8613;
        pos:=ldat.Search(@d);
        if pos < 0 then
          ShowMessage('The item '+ IntToStr(d.code) + ' is not found')
        else
        dat:=ldat.Items[pos];
  // Clearing of the list
        ldat.ClearAll;
...
end;
...
end;
...
initialization
  ldat:=TSortList.Create;
  ldat.Compare:=Sort_dat;
finalization
  ldat.Free;
end.
*********************************************************************}
unit SortList;

interface
uses Classes, SysUtils;

type
     TSortList = class(TList)
     private
       Ret: integer;
       ERR: byte;
       pcl,pcr: Pointer;
       FCompare: TListSortCompare;
       procedure SetCompare(Value: TListSortCompare);
       function SearchItem(Item: Pointer): integer;
       procedure QuickSearch(Item: Pointer; L,R: integer);
     public
       procedure AddSort(Item: Pointer);
       function  Search(Item: Pointer):integer;
       procedure ClearAll;
       property  Compare: TListSortCompare read FCompare write SetCompare;
end;

implementation
//*******************************************
procedure TSortList.ClearAll;
var
i: integer;
Item: Pointer;
begin
  if Count <> 0 then
    for i:=0 to Count-1 do
    begin
      item:=Items[i];
      try
        Dispose(Item);
      finally end;
    end;
  Clear;
end;
//------------------------------------------------------
procedure TSortList.SetCompare(Value: TListSortCompare);
begin
   FCompare:=Value;
end;
//-----------------------------------------------------------
procedure TSortList.QuickSearch(Item: Pointer; L,R: integer);
var
  K: Integer;
  P: Pointer;
begin
   ERR:=0;
   Ret:=-1;
   pcl:=Items[L];
   if Compare(Item,pcl) < 0 then
   begin
     Ret:=L;
     ERR:=1;
     exit;
   end
   else
   if Compare(Item,pcl) = 0 then
   begin
     Ret:=L;
     exit;
   end;
   pcr:=Items[R];
   if Compare(Item,pcr) > 0 then
   begin
     Ret:=R;
     ERR:=2;
     exit;
   end
   else
   if Compare(Item,pcr) = 0 then
   begin
     Ret:=R;
     exit;
   end;
//-----------------
   if R-L > 1 then
   begin
     K:=(R-L) div 2;
     P:=items[L+K];
     if Compare(Item,P) < 0 then
	QuickSearch(Item,L,L+K)
     else
     begin
      if Compare(Item,P) > 0 then
	QuickSearch(Item,L+K,R)
      else
	if Compare(Item,P) = 0 then
	begin
	   Ret:=L+K;
	   exit;
	end;
     end;
   end
   else
   begin
     ERR:=1;
     ret:=R;
   end;
end;
//----------------------------------------------------
function TSortList.SearchItem(Item: Pointer): integer;
begin
    if Count > 0 then
    begin
      QuickSearch(Item, 0, Count - 1);
      Result:=Ret;
    end
    else
    begin
	Result:=0;
	ERR:=2;
    end;
end;
//------------------------------------------------
function TSortList.Search(Item: Pointer): integer;
begin
    if Addr(Compare)=nil then
    begin
      Error('The function of comparing is not assigned',-1);
      Result:=-1;
      exit;
    end;
    Result:=SearchItem(item);
    if ERR <> 0 then Result:=-1;
end;
//-----------------------------------------
procedure TSortList.AddSort(item: Pointer);
var
i: integer;
begin
    if Addr(Compare)=nil then
    begin
      Error('The function of comparing is not assigned',-1);
      exit;
    end;
    i:=SearchItem(item);
    if (ERR=0) or (ERR=1) then  Insert(i,item)
    else if ERR=2 then Add(item);
end;
end.
